@@ -19,12 +19,17 @@
  * Boston, MA 02111-1307, USA.
  */
 #include "config.h"
-#include <cairo-ft.h>
 
-#include "pangofc-fontmap.h"
+/* FreeType has undefined macros in its headers */
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wundef"
+#include <cairo-ft.h>
+#pragma GCC diagnostic pop
+
+#include "pangofc-fontmap-private.h"
 #include "pangocairo.h"
 #include "pangocairo-private.h"
-#include "pangocairo-fc.h"
+#include "pangocairo-fc-private.h"
 
 typedef struct _PangoCairoFcFontMapClass PangoCairoFcFontMapClass;
 
@@ -41,6 +46,16 @@ pango_cairo_fc_font_map_get_serial (PangoFontMap *fontmap)
   return cffontmap->serial;
 }
 
+static void
+pango_cairo_fc_font_map_changed (PangoFontMap *fontmap)
+{
+  PangoCairoFcFontMap *cffontmap = (PangoCairoFcFontMap *) (fontmap);
+
+  cffontmap->serial++;
+  if (cffontmap->serial == 0)
+    cffontmap->serial++;
+}
+
 static void
 pango_cairo_fc_font_map_set_resolution (PangoCairoFontMap *cfontmap,
 					double             dpi)
@@ -81,25 +96,17 @@ cairo_font_map_iface_init (PangoCairoFontMapIface *iface)
 }
 
 G_DEFINE_TYPE_WITH_CODE (PangoCairoFcFontMap, pango_cairo_fc_font_map, PANGO_TYPE_FC_FONT_MAP,
-    { G_IMPLEMENT_INTERFACE (PANGO_TYPE_CAIRO_FONT_MAP, cairo_font_map_iface_init) })
-
-static void
-pango_cairo_fc_font_map_finalize (GObject *object)
-{
-  PangoCairoFcFontMap *cffontmap = (PangoCairoFcFontMap *) (object);
-
-  G_OBJECT_CLASS (pango_cairo_fc_font_map_parent_class)->finalize (object);
-
-  FT_Done_FreeType (cffontmap->library);
-}
+                         G_IMPLEMENT_INTERFACE (PANGO_TYPE_CAIRO_FONT_MAP, cairo_font_map_iface_init))
 
 static void
 pango_cairo_fc_font_map_fontset_key_substitute (PangoFcFontMap    *fcfontmap G_GNUC_UNUSED,
 						PangoFcFontsetKey *fontkey,
 						FcPattern         *pattern)
 {
-  FcConfigSubstitute (NULL, pattern, FcMatchPattern);
+  FcConfigSubstitute (pango_fc_font_map_get_config (fcfontmap), pattern, FcMatchPattern);
 
+  if (fcfontmap->substitute_func)
+    fcfontmap->substitute_func (pattern, fcfontmap->substitute_data);
   if (fontkey)
     cairo_ft_font_options_substitute (pango_fc_fontset_key_get_context_key (fontkey),
 				      pattern);
@@ -175,13 +182,11 @@ pango_cairo_fc_font_map_create_font (PangoFcFontMap *fcfontmap,
 static void
 pango_cairo_fc_font_map_class_init (PangoCairoFcFontMapClass *class)
 {
-  GObjectClass *gobject_class = G_OBJECT_CLASS (class);
   PangoFontMapClass *fontmap_class = PANGO_FONT_MAP_CLASS (class);
   PangoFcFontMapClass *fcfontmap_class = PANGO_FC_FONT_MAP_CLASS (class);
 
-  gobject_class->finalize  = pango_cairo_fc_font_map_finalize;
-
   fontmap_class->get_serial = pango_cairo_fc_font_map_get_serial;
+  fontmap_class->changed = pango_cairo_fc_font_map_changed;
 
   fcfontmap_class->fontset_key_substitute = pango_cairo_fc_font_map_fontset_key_substitute;
   fcfontmap_class->get_resolution = pango_cairo_fc_font_map_get_resolution_fc;
@@ -198,22 +203,6 @@ pango_cairo_fc_font_map_class_init (PangoCairoFcFontMapClass *class)
 static void
 pango_cairo_fc_font_map_init (PangoCairoFcFontMap *cffontmap)
 {
-  FT_Error error;
-
-  cffontmap->library = NULL;
-  error = FT_Init_FreeType (&cffontmap->library);
-  if (error != FT_Err_Ok)
-    g_critical ("pango_cairo_font_map_init: Could not initialize freetype");
-
   cffontmap->serial = 1;
   cffontmap->dpi   = 96.0;
 }
-
-FT_Library
-_pango_cairo_fc_font_map_get_library (PangoCairoFcFontMap *fontmap)
-{
-  g_return_val_if_fail (PANGO_IS_CAIRO_FC_FONT_MAP (fontmap), NULL);
-
-  return fontmap->library;
-}
-
