@@ -142,6 +142,8 @@ double                pango_core_text_font_key_get_resolution       (const Pango
 _PANGO_EXTERN
 gboolean              pango_core_text_font_key_get_synthetic_italic (const PangoCoreTextFontKey *key);
 _PANGO_EXTERN
+gboolean              pango_core_text_font_key_get_synthetic_small_caps (const PangoCoreTextFontKey *key);
+_PANGO_EXTERN
 gpointer              pango_core_text_font_key_get_context_key      (const PangoCoreTextFontKey *key);
 _PANGO_EXTERN
 const PangoMatrix    *pango_core_text_font_key_get_matrix           (const PangoCoreTextFontKey *key);
@@ -149,6 +151,8 @@ _PANGO_EXTERN
 PangoGravity          pango_core_text_font_key_get_gravity          (const PangoCoreTextFontKey *key);
 _PANGO_EXTERN
 CTFontDescriptorRef   pango_core_text_font_key_get_ctfontdescriptor (const PangoCoreTextFontKey *key);
+_PANGO_EXTERN
+const char *          pango_core_text_font_key_get_variations       (const PangoCoreTextFontKey *key);
 
 PangoCoreTextFace *   pango_core_text_font_map_find_face (PangoCoreTextFontMap       *map,
                                                           const PangoCoreTextFontKey *key);
