@@ -126,6 +126,16 @@ test_parse (gconstpointer d)
   test_file (filename, string);
 
   diff = diff_with_file (expected_file, string->str, string->len, &error);
+  if (g_error_matches (error, G_SPAWN_ERROR, G_SPAWN_ERROR_NOENT))
+    {
+      g_test_skip ("diff binary not found");
+
+      g_string_free (string, TRUE);
+      g_free (expected_file);
+
+      return;
+    }
+
   g_assert_no_error (error);
 
   if (diff && diff[0])
@@ -136,7 +146,6 @@ test_parse (gconstpointer d)
   g_free (diff);
 
   g_string_free (string, TRUE);
-
   g_free (expected_file);
 }
 
@@ -238,6 +247,16 @@ test_parse_incrementally (gconstpointer d)
   else
     {
       diff = diff_with_file (expected_file, string->str, string->len, &error);
+      if (g_error_matches (error, G_SPAWN_ERROR, G_SPAWN_ERROR_NOENT))
+        {
+          g_test_skip ("diff binary not found");
+
+          g_string_free (string, TRUE);
+          g_free (expected_file);
+
+          return;
+        }
+
       g_assert_no_error (error);
 
       if (diff && diff[0])
@@ -261,7 +280,7 @@ main (int argc, char *argv[])
   const gchar *name;
   gchar *path;
 
-  g_setenv ("LC_ALL", "C", TRUE);
+  g_setenv ("LC_ALL", "en_US.UTF8", TRUE);
   setlocale (LC_ALL, "");
 
   /* allow to easily generate expected output for new test cases */
