@@ -20,6 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
 #include <pango/pango.h>
 #include <stdlib.h>
 #include <string.h>
@@ -382,6 +383,10 @@ test_line_break (void)
   const char *filename;
   AttrBits bits;
 
+#ifdef HAVE_LIBTHAI
+  g_test_skip ("Compiled with libthai, skip the line break test.");
+  return;
+#endif
 
   filename = g_test_get_filename (G_TEST_DIST, "LineBreakTest.txt", NULL);
   bits.bits = 0;
@@ -399,6 +404,7 @@ main (gint argc,
   setlocale (LC_ALL, "");
 
   g_test_init (&argc, &argv, NULL);
+  g_test_set_nonfatal_assertions ();
 
   g_test_add_func ("/text/break/grapheme", test_grapheme_break);
   g_test_add_func ("/text/break/word", test_word_break);
