@@ -60,18 +60,6 @@
 
 #include "pango/pango-script.h"
 
-#undef VERBOSE
-
-#define ASSERT(stmt) G_STMT_START {					\
-    if (stmt) { }							\
-    else								\
-      {									\
-	g_warning ("%s:%d (%s): assertion '%s' failed",			\
-		 __FILE__, __LINE__, G_STRFUNC, #stmt);			\
-	exit (1);							\
-      }									\
-} G_STMT_END
-
 typedef struct
 {
   const char *run_text_escaped;
@@ -155,6 +143,7 @@ test_script_iter (void)
   };
 
   PangoScriptIter *iter;
+  PangoScriptIter *iter2;
   GString *all = g_string_new (FALSE);
   char *pos;
   const char *start;
@@ -170,9 +159,9 @@ test_script_iter (void)
 
   iter = pango_script_iter_new (all->str, -1);
 
-#ifdef VERBOSE
-  g_print ("Total length: %d\n", all->len);
-#endif
+  iter2 = g_boxed_copy (pango_script_iter_get_type (), iter);
+
+  g_test_message ("Total length: %" G_GSIZE_FORMAT "\n", all->len);
 
   pos = all->str;
   for (i = 0; i < G_N_ELEMENTS(test_data); i++)
@@ -181,20 +170,29 @@ test_script_iter (void)
       gboolean result;
 
       pango_script_iter_get_range (iter, &start, &end, &script);
-#ifdef VERBOSE
-      g_print ("Range: %d-%d: %d\n", start - all->str, end - all->str, script);
-#endif
 
-      ASSERT (start == pos);
-      ASSERT (end == next_pos);
-      ASSERT (script == test_data[i].run_code);
+      g_test_message ("Range: %d-%d: %d\n",
+                      (int) (start - all->str),
+                      (int) (end - all->str),
+                      script);
+
+      g_assert_true (start == pos);
+      g_assert_true (end == next_pos);
+      g_assert_true (script == test_data[i].run_code);
 
       result = pango_script_iter_next (iter);
-      ASSERT (result == (i != G_N_ELEMENTS (test_data) - 1));
+      g_assert_true (result == (i != G_N_ELEMENTS (test_data) - 1));
 
       pos = next_pos;
     }
 
+  /* Check that copying the iter worked */
+  pango_script_iter_get_range (iter2, &start, &end, &script);
+  g_assert_true (start == all->str);
+  g_assert_true (end == all->str + strlen (test_data[0].run_text));
+  g_assert_true (script == test_data[0].run_code);
+  pango_script_iter_free (iter2);
+
   pango_script_iter_free (iter);
 
   /*
@@ -204,10 +202,10 @@ test_script_iter (void)
 
   pango_script_iter_get_range (iter, &start, &end, &script);
 
-  ASSERT (start == all->str);
-  ASSERT (end == all->str);
-  ASSERT (script == PANGO_SCRIPT_COMMON);
-  ASSERT (!pango_script_iter_next (iter));
+  g_assert_true (start == all->str);
+  g_assert_true (end == all->str);
+  g_assert_true (script == PANGO_SCRIPT_COMMON);
+  g_assert_true (!pango_script_iter_next (iter));
 
   pango_script_iter_free (iter);
 
@@ -222,9 +220,9 @@ test_script_iter (void)
 int
 main (int argc, char **argv)
 {
-  g_setenv ("PANGO_RC_FILE", "./pangorc", TRUE);
+  g_test_init (&argc, &argv, NULL);
 
-  test_script_iter ();
+  g_test_add_func ("/script/iter", test_script_iter);
 
-  return 0;
+  return g_test_run ();
 }
